var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
import { spread, effect, className, template } from "solid-js/web";
import { createSignal, createMemo, createEffect, onCleanup, on, splitProps, mergeProps } from "solid-js";
import { EditorState, StateEffect } from "@codemirror/state";
import { EditorView, keymap, placeholder } from "@codemirror/view";
import { indentWithTab } from "@codemirror/commands";
import { createStore } from "solid-js/store";
const updateListenerExtension = (onChange) => {
  return EditorView.updateListener.of((vu) => {
    if (vu.docChanged && typeof onChange === "function") {
      const document = vu.state.doc;
      const value = document.toString();
      onChange(value, vu);
    }
  });
};
function createCodeMirror(initialOptions) {
  const [options, setOptions] = createStore(initialOptions);
  const [container, setContainer] = createSignal();
  const [view, setView] = createSignal();
  const [state, setState] = createSignal();
  const defaultThemeOption = EditorView.theme({
    "&": {
      height: initialOptions.height,
      minHeight: initialOptions.minHeight,
      maxHeight: initialOptions.maxHeight,
      width: initialOptions.width,
      minWidth: initialOptions.minWidth,
      maxWidth: initialOptions.maxWidth
    }
  });
  const computedExtensions = createMemo(() => {
    const {
      indentWithTab: indentWithTab$1,
      placeholder: placeholder$1,
      editable,
      onUpdate,
      extensions
    } = options;
    const updateListener = updateListenerExtension(options.onChange);
    return [
      indentWithTab$1 ? keymap.of([indentWithTab]) : null,
      placeholder$1 && placeholder ? placeholder(placeholder$1) : null,
      !editable ? EditorView.editable.of(false) : null,
      !!onUpdate && typeof onUpdate === "function" ? EditorView.updateListener.of(onUpdate) : null,
      updateListener,
      defaultThemeOption,
      ...extensions || null
    ].filter(Boolean);
  }, options);
  createEffect(() => {
    const { value, selection, root } = options;
    if (container() && !state()) {
      const stateCurrent = EditorState.create({
        doc: value,
        selection,
        extensions: computedExtensions()
      });
      setState(stateCurrent);
      if (!view()) {
        const viewCurrent = new EditorView({
          state: stateCurrent,
          parent: container(),
          root
        });
        setView(viewCurrent);
      }
    }
  });
  onCleanup(() => {
    var _a;
    return (_a = view()) == null ? void 0 : _a.destroy();
  });
  createEffect(() => {
    var _a, _b;
    if (view()) {
      const currentValue = (_a = view()) == null ? void 0 : _a.state.doc.toString();
      if (options.value !== currentValue) {
        (_b = view()) == null ? void 0 : _b.dispatch({
          changes: {
            from: 0,
            to: currentValue == null ? void 0 : currentValue.length,
            insert: options.value || ""
          }
        });
      }
    }
  });
  createEffect(on(computedExtensions, () => {
    var _a;
    return (_a = view()) == null ? void 0 : _a.dispatch({
      effects: StateEffect.reconfigure.of(computedExtensions())
    });
  }));
  createEffect(() => {
    var _a;
    if (options.autoFocus && view()) {
      (_a = view()) == null ? void 0 : _a.focus();
    }
  });
  return {
    options,
    setOptions,
    state,
    setState,
    view,
    setView,
    container,
    setContainer
  };
}
const extractCodeMirrorProps = (props) => {
  const [
    options,
    htmlAttrs
  ] = splitProps(props, [
    "class",
    "value",
    "selection",
    "onChange",
    "onUpdate",
    "autoFocus",
    "height",
    "minHeight",
    "maxHeight",
    "width",
    "minWidth",
    "maxWidth",
    "placeholder",
    "editable",
    "root",
    "extensions",
    "indentWithTab"
  ]);
  const computedOptions = mergeProps({
    class: "",
    extensions: [],
    height: "auto",
    minHeight: "none",
    maxHeight: "none",
    placeHolder: "",
    width: "auto",
    minWidth: "none",
    maxWidth: "none",
    editable: true,
    indentWithTab: true,
    basicSetup: true
  }, options);
  return { options: computedOptions, htmlAttrs };
};
const _tmpl$ = /* @__PURE__ */ template(`<div></div>`, 2);
const CodeMirror = (props) => {
  let editor;
  const {
    options,
    htmlAttrs
  } = extractCodeMirrorProps(props);
  const {
    view,
    setOptions,
    setContainer
  } = createCodeMirror(__spreadValues({
    container: editor
  }, options));
  createEffect(() => setContainer(editor));
  createEffect(() => setOptions(options));
  onCleanup(() => {
    var _a;
    return (_a = view()) == null ? void 0 : _a.destroy();
  });
  return (() => {
    const _el$ = _tmpl$.cloneNode(true);
    ((ref) => editor = ref)(_el$);
    spread(_el$, htmlAttrs, false, false);
    effect(() => className(_el$, `solid-cm ${(options == null ? void 0 : options.class) ? ` ${options.class}` : ""}`));
    return _el$;
  })();
};
export { CodeMirror, createCodeMirror };
